'***************************************************************************
'
'ATTiny13 dust sensor GP2U06 controller
'
' dust sensor pulse output PB0
' input on ADC channel 1
'
'***************************************************************************
'  This program is free software; you can redistribute it and/or
'  modify it under the terms of the GNU General Public License.
'  This program is distributed in the hope that it will be useful,
'  but WITHOUT ANY WARRANTY;
'
'***************************************************************************
'
'  Pinout ATtiny13/ATtiny13V   8-PDIP/SOIC
'
' (PCINT5/RESET/ADC0/dW)   PB5 VCC
' (PCINT3/CLKI/ADC3)       PB3 PB2 (SCK/ADC1/T0/PCINT2)
' (PCINT4/ADC2)            PB4 PB1 (MISO/AIN1/OC0B/INT0/PCINT1)
'                          GND PB0 (MOSI/AIN0/OC0A/PCINT0)
'
'***************************************************************************
' PB0 output  dust pulse 0.4 ms
' PB1 output  Ipulse
' PB2 input   ADC channel 1
' PB3 output  Led1 hi
' PB4 output  RS232
'***************************************************************************

$regfile = "attiny13.dat"
$crystal = 1200000
$hwstack = 8
$swstack = 8
$framesize = 4

Const C_value = 187 - 1                                     ' Compare Match IRQ 10 ms start
Declare Sub Mydelay

Dpulse Alias Portb.0
Ipulse Alias Portb.1
Led1 Alias Portb.3

Dim Voltage As Word , Avgv As Word
Dim Icnt0 As Byte , Icnt1 As Byte

Portb = &B00000000
Ddrb = &B0011011

Acsr.acd = 0                                                ' switch off analog comparator

Config Adc = Single , Prescaler = Auto , Reference = Avcc
Didr0.adc1d = 1
Start Adc

Config Timer0 = Timer , Prescale = 64
Ocr0a = C_value                                             ' value for 1.2 MHz
Tccr0a = &B00000010                                         ' ctc
Timsk0.ocie0a = 1
On Oc0a Tim0_compa
Start Timer0

Open "comb.4:9600,8,n,1,INVERTED" For Output As #1


Enable Interrupts


Do                                                          'main loop

  If Icnt1 >= 40 Then                                       '400 ms
'     Avgv = Avgv / 40
     Print #1 , Voltage
     Icnt1 = 0
     Avgv = 0
  End If

  If Voltage > 100 Then                                     'turn on Led
     Led1 = 1
  Else
     Led1 = 0
  End If

Loop


'***************************************************************************
'Tim0_compa Interrupt  10 ms
'***************************************************************************

Tim0_compa:                                                 '

  Ipulse = 1
  Incr Icnt0
  Incr Icnt1
  If Icnt0 >= 3 Then
    Dpulse = 1                                              '470 us pulse total
    Call Mydelay
    Voltage = Getadc(1)
    Avgv = Avgv + Voltage
    'Shift Voltage , Right , 2                               '8 bit
    Call Mydelay
    Dpulse = 0
    Icnt0 = 0
  End If
  Ipulse = 0

Return

End

Mydelay:                                                    'delay
$asm
   PUSH R27
   Ldi R27 , 39                                             'Load R27
Mdec:
   Dec R27
   BRNE Mdec
   POP R27
$END ASM

Return